@@ -1,4 +1,3 @@
-import io
 import os
 import pathlib
 import sys
@@ -51,6 +50,10 @@ class TestSimpleModule:
         assert foo.find(id="example.Foo.Meta")
 
         # Check that class attributes are documented
+        attr = foo.find(id="example.Foo.attr")
+        assert attr
+        assert attr.find(class_="pre").text.strip() == "property"
+
         attr2 = foo.find(id="example.Foo.attr2")
         assert "attr2" in attr2.text
         # Check that attribute docstrings are used
@@ -94,6 +97,10 @@ class TestSimpleModule:
             == "This property should parse okay."
         )
 
+        my_cached_property = foo.find(id="example.Foo.my_cached_property")
+        assert my_cached_property
+        assert my_cached_property.find(class_="pre").text.strip() == "property"
+
         # Overridden methods without their own docstring
         # should inherit the parent's docstring
         bar_method_okay = example_file.find(id="example.Bar.method_okay")
@@ -137,9 +144,9 @@ class TestSimpleModule:
     def test_show_inheritance(self, parse):
         example_file = parse("_build/html/autoapi/example/index.html")
 
-        foo = example_file.find(id="example.Foo")
-        foo_docstring = foo.parent.find("dd").contents[0]
-        assert foo_docstring.text.startswith("Bases:")
+        bar = example_file.find(id="example.Bar")
+        bar_docstring = bar.parent.find("dd").contents[0]
+        assert bar_docstring.text.startswith("Bases:")
 
     def test_long_signature(self, parse):
         example_file = parse("_build/html/autoapi/example/index.html")
@@ -269,6 +276,44 @@ class TestPy3Module:
         assert "Initialize self" not in example_file
         assert "a new type" not in example_file
 
+    def test_inheritance(self, parse):
+        example_file = parse("_build/html/autoapi/example/index.html")
+
+        # Test that members are not inherited from standard library classes.
+        assert example_file.find(id="example.MyException")
+        assert not example_file.find(id="example.MyException.args")
+        # With the exception of abstract base classes.
+        assert example_file.find(id="example.My123")
+        assert example_file.find(id="example.My123.__contains__")
+        assert example_file.find(id="example.My123.index")
+
+        # Test that classes inherit instance attributes
+        exc = example_file.find(id="example.InheritError")
+        assert exc
+        message = example_file.find(id="example.InheritError.my_message")
+        assert message
+        message_docstring = message.parent.find("dd").text.strip()
+        assert message_docstring == "My message."
+
+        # Test that classes inherit from the whole mro
+        exc = example_file.find(id="example.SubInheritError")
+        assert exc
+        message = example_file.find(id="example.SubInheritError.my_message")
+        message_docstring = message.parent.find("dd").text.strip()
+        assert message_docstring == "My message."
+        message = example_file.find(id="example.SubInheritError.my_other_message")
+        assert message
+        message_docstring = message.parent.find("dd").text.strip()
+        assert message_docstring == "My other message."
+
+        # Test that inherited instance attributes include the docstring
+        exc = example_file.find(id="example.DuplicateInheritError")
+        assert exc
+        message = example_file.find(id="example.DuplicateInheritError.my_message")
+        assert message
+        message_docstring = message.parent.find("dd").text.strip()
+        assert message_docstring == "My message."
+
     def test_annotations(self, parse):
         example_file = parse("_build/html/autoapi/example/index.html")
 
@@ -813,7 +858,7 @@ class _CompareInstanceType:
         return self.type is type(other)
 
     def __repr__(self):
-        return "<expect type {}>".format(self.type.__name__)
+        return f"<expect type {self.type.__name__}>"
 
 
 def test_skip_members_hook(builder):
@@ -1100,7 +1145,7 @@ def test_string_module_attributes(builder):
     builder("py3example", confoverrides=keep_rst)
 
     example_path = os.path.join("autoapi", "example", "index.rst")
-    with io.open(example_path, encoding="utf8") as example_handle:
+    with open(example_path, encoding="utf8") as example_handle:
         example_file = example_handle.read()
 
     code_snippet_contents = [
@@ -1190,3 +1235,58 @@ class TestMemberOrder:
         method_sphinx_docs = example_file.find(id="example.Foo.method_sphinx_docs")
 
         assert method_tricky.sourceline < method_sphinx_docs.sourceline
+
+
+def test_nothing_to_render_raises_warning(builder):
+    with pytest.raises(sphinx.errors.SphinxWarning) as exc_info:
+        builder("pynorender", warningiserror=True)
+
+    assert "No modules were rendered" in str(exc_info.value)
+
+
+class TestStdLib:
+    """Check that modules with standard library names are still documented."""
+
+    @pytest.fixture(autouse=True)
+    def built(self, builder):
+        builder("pystdlib")
+
+    def test_integration(self, parse):
+        json_file = parse("_build/html/autoapi/json/index.html")
+
+        json_mod = json_file.find(id="json")
+        assert "json is the same name" in json_mod.parent.text
+
+        assert json_file.find(id="json.JSONDecoder")
+        decode = json_file.find(id="json.JSONDecoder.decode")
+        assert decode
+        wrap_docstring = decode.parent.find("dd").text.strip()
+        assert wrap_docstring == "Decode a string."
+
+        myjson_file = parse("_build/html/autoapi/myjson/index.html")
+
+        # Find members that are not inherited from local standard library classes.
+        ctx = myjson_file.find(id="myjson.MyJSONDecoder")
+        assert ctx
+        meth = myjson_file.find(id="myjson.MyJSONDecoder.my_method")
+        assert meth
+        meth_docstring = meth.parent.find("dd").text.strip()
+        assert meth_docstring == "This is a method."
+
+        # Find members that are inherited from local standard library classes.
+        decode = myjson_file.find(id="myjson.MyJSONDecoder.decode")
+        assert decode
+
+        myast_file = parse("_build/html/autoapi/myast/index.html")
+
+        # Find members that are not inherited from standard library classes.
+        visitor = myast_file.find(id="myast.MyVisitor")
+        assert visitor
+        meth = myast_file.find(id="myast.MyVisitor.my_visit")
+        assert meth
+        meth_docstring = meth.parent.find("dd").text.strip()
+        assert meth_docstring == "My visit method."
+
+        # Find members that are inherited from standard library classes.
+        meth = myast_file.find(id="myast.MyVisitor.visit")
+        assert not meth
